#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "scramble.h"
#include "encode.h"

int
encode (long orig, long *enc1, long *enc2)
{
  long tmp1, tmp2;

  *enc1 = orig;
  *enc2 = orig ^ CHECK_CONST;

  *enc1 += ENC1_CONST;
  *enc2 += ENC2_CONST;

  scramble (tmp1, tmp2, *enc1, *enc2);
  *enc1 = tmp1;
  *enc2 = tmp2;

  // 6 Bits from 0 -- Down
  tmp1 = *enc1 & ((1L<<6)-1);
  gen_loop_down (tmp1, MASK1, COUNT1, MSB1, tmp2);
  *enc1 = (~((1L<<6)-1) & *enc1) | tmp1;

  // 17 Bits from 6 -- Up
  tmp1 = (*enc1>>6) & ((1L<<17)-1);
  gen_loop_up (tmp1, MASK2, COUNT2, tmp2);
  *enc1 =  (~(((1L<<17)-1)<<6) & *enc1) | (tmp1<<6);

  // 25 bits from 23 -- Down
  // (9 bits from enc1 at 23, 16 bits from enc2 at 0)
  tmp1 = ((*enc1>>23)&0x1FF) | ((*enc2<<9)&0x01FFFE00L);
  gen_loop_down (tmp1, MASK3, COUNT3, MSB3, tmp2);
  *enc1 = (*enc1 & 0x007FFFFFL) | ((tmp1<<23)& ~0x007FFFFFL);
  *enc2 = (*enc2 & 0xFFFF0000L) | ((tmp1>> 9)& ~0xFFFF0000L);

  // 9 bits from 48 (enc2, bit 16) -- Down
  tmp1 = (*enc2>>16) & ((1L<<9)-1);
  gen_loop_down (tmp1, MASK4, COUNT4, MSB4, tmp2);
  *enc2 =  (~(((1L<<9)-1)<<16) & *enc2) | (tmp1<<16);

  // 7 bits from 57 (enc2, bit 25) -- Up
  tmp1 = (*enc2>>25) & ((1L<<7)-1);
  gen_loop_up (tmp1, MASK5, COUNT5, tmp2);
  *enc2 =  (~(((1L<<7)-1)<<25) & *enc2) | (tmp1<<25);


  return (0);
}


int
decode (long *orig, long enc1, long enc2)
{
  long tmp1, tmp2;

  // 6 Bits from 0 -- Down
  tmp1 = (enc1>>0) & ((1L<<6)-1);
  gen_loop_up (tmp1, MASK1, COUNT1, tmp2);
  enc1 = (~((1L<<6)-1) & enc1) | tmp1;

  // 17 Bits from 6 -- Up
  tmp1 = (enc1>>6) & ((1L<<17)-1);
  gen_loop_down (tmp1, MASK2, COUNT2, MSB2, tmp2);
  enc1 =  (~(((1L<<17)-1)<<6) & enc1) | (tmp1<<6);

  // 25 bits from 23 -- Down
  // (9 bits from enc1 at 23, 16 bits from enc2 at 0)
  tmp1 = ((enc1>>23)&0x1FF) | ((enc2<<9)&0x01FFFE00L);
  gen_loop_up (tmp1, MASK3, COUNT3, tmp2);
  enc1 = (enc1 & 0x007FFFFFL) | ((tmp1<<23)& ~0x007FFFFFL);
  enc2 = (enc2 & 0xFFFF0000L) | ((tmp1>> 9)& ~0xFFFF0000L);

  // 9 bits from 48 (enc2, bit 16) -- Down
  tmp1 = (enc2>>16) & ((1L<<9)-1);
  gen_loop_up (tmp1, MASK4, COUNT4, tmp2);
  enc2 =  (~(((1L<<9)-1)<<16) & enc2) | (tmp1<<16);

  // 7 bits from 57 (enc2, bit 25) -- Up
  tmp1 = (enc2>>25) & ((1L<<7)-1);
  gen_loop_down (tmp1, MASK5, COUNT5, MSB5, tmp2);
  enc2 =  (~(((1L<<7)-1)<<25) & enc2) | (tmp1<<25);

  unscramble (tmp1, tmp2, enc1, enc2);
  enc1=tmp1;
  enc2=tmp2;

  enc1 -= ENC1_CONST;
  enc2 -= ENC2_CONST;

  enc2 ^= CHECK_CONST;

  if (enc1==enc2)
    {
      *orig = enc1;
      return (0);
    }
  else
    {
      *orig = 0;
      return (1);
    }
}

